#+ELEVENTY_COLLECTIONS: _posts
#+ELEVENTY_BASE_DIR: ~/proj/static-blog/
#+ELEVENTY_CATEGORIES+: emacs
#+ELEVENTY_LAYOUT: layouts/post

* Running the current Org Mode Babel Javascript block from Emacs using Spookfox :emacs:org:spookfox:
:PROPERTIES:
:CUSTOM_ID: spookfox-babel
:EXPORT_DATE: 2024-01-19T08:12:17-0500
:EXPORT_ELEVENTY_PERMALINK: /blog/2024/01/running-the-current-org-mode-babel-javascript-block-in-spookfox/
:EXPORT_ELEVENTY_FILE_NAME: blog/2024/01/running-the-current-org-mode-babel-javascript-block-in-spookfox/
:END:

I often want to send Javascript from Emacs to the web browser. It's handy for testing code snippets or working with data on pages that require Javascript or authentication. I could start Google Chrome or Mozilla Firefox with their remote debugging protocols, copy the websocket URLs, and talk to the browser through  something like Puppeteer, but it's so much easier to use the [[https://github.com/bitspook/spookfox][Spookfox]] extension for Mozilla to execute code in the active tab.
~spookfox-js-injection-eval-in-active-tab~ lets you evaluate Javascript and get the results back in Emacs Lisp.

I wanted to be able to execute code even more
easily. This code lets me add a ~:spookfox t~
parameter to Org Babel Javascript blocks so that I
can run the block in my Firefox active tab.
For example, if I have ~(spookfox-init)~ set up, Spookfox connected, and https://planet.emacslife.com in my active tab, I can use it with the following code:

#+begin_src org
,#+begin_src js :eval never-export :spookfox t :exports results
[...document.querySelectorAll('.post > h2')].slice(0,5).map((o) => '- ' + o.textContent.trim().replace(/[ \n]+/g, ' ') + '\n').join('')
,#+end_src
#+end_src

#+begin_src js :eval never-export :spookfox t :exports results
[...document.querySelectorAll('.post > h2')].slice(0,5).map((o) => '- ' + o.textContent.trim().replace(/[ \n]+/g, ' ') + '\n').join('')
#+end_src

#+RESULTS:
:results:
- Mario Jason Braganza: Updated to Emacs 29.2
- Irreal: Zamansky: Learning Elisp #16
- Tim Heaney: Lisp syntax
- Erik L. Arneson: Many Posts of Interest for January 2024
- William Denton: Basic citations in Org (Part 4)
:end:

[[video:/home/sacha/recordings/2024-01-19_08.16.31.webm?caption=Evaluating a Javascript block with :spookfox t]]

To do this, we wrap some advice around the ~org-babel-execute:js~ function that's called by ~org-babel-execute-src-block~.

#+begin_src emacs-lisp
(defun my-org-babel-execute:js-spookfox (old-fn body params)
	"Maybe execute Spookfox."
	(if (assq :spookfox params)
			(spookfox-js-injection-eval-in-active-tab
			 body t)
		(funcall old-fn body params)))
(with-eval-after-load 'ob-js
	(advice-add 'org-babel-execute:js :around #'my-org-babel-execute:js-spookfox))
#+end_src

I can also run the block in Spookfox without adding the parameter if I make an interactive function:

#+begin_src emacs-lisp
(defun my-spookfox-eval-org-block ()
	(interactive)
	(let ((block (org-element-context)))
		(when (and (eq (org-element-type block) 'src-block)
							 (string= (org-element-property :language block) "js"))
			(spookfox-js-injection-eval-in-active-tab
			 (nth 2 (org-src--contents-area block))
			 t))))
#+end_src

I can add that as an Embark context action:

#+begin_src emacs-lisp
(with-eval-after-load 'embark-org
	(define-key embark-org-src-block-map "f" #'my-spookfox-eval-org-block))
#+end_src

In Javascript buffers, I want the ability to send the current line, region, or buffer too, just like package:nodejs-repl does.

#+begin_src emacs-lisp
(defun my-spookfox-send-region (start end)
	(interactive "r")
	(spookfox-js-injection-eval-in-active-tab (buffer-substring start end) t))

(defun my-spookfox-send-buffer ()
	(interactive)
	(my-spookfox-send-region (point-min) (point-max)))

(defun my-spookfox-send-line ()
	(interactive)
	(my-spookfox-send-region (line-beginning-position) (line-end-position)))

(defun my-spookfox-send-last-expression ()
	(interactive)
	(my-spookfox-send-region (save-excursion (nodejs-repl--beginning-of-expression)) (point)))

(defvar-keymap my-js-spookfox-minor-mode-map
	:doc "Send parts of the buffer to Spookfox."
	"C-x C-e" 'my-spookfox-send-last-expression
	"C-c C-j" 'my-spookfox-send-line
	"C-c C-r" 'my-spookfox-send-region
	"C-c C-c" 'my-spookfox-send-buffer)

(define-minor-mode my-js-spookfox-minor-mode "Send code to Spookfox.")
#+end_src

I usually edit Javascript files with package:js2-mode, so I can use ~my-js-spookfox-minor-mode~ in addition to that.

I can turn the minor mode on automatically for ~:spookfox t~ source blocks. There's no ~org-babel-edit-prep:js~ yet, I think, so we need to define it instead of advising it.

#+begin_src emacs-lisp
(defun org-babel-edit-prep:js (info)
	(when (assq :spookfox (nth 2 info))
		(my-js-spookfox-minor-mode 1)))
#+end_src

Let's try it out by sending the last line repeatedly:

#+begin_src js :eval never-export :spookfox t :exports results
window.location = 'https://example.com';
document.querySelector('h1').style.fontSize = Math.floor(Math.random() * 10 + 10) + 'px';
#+end_src

[[video:/home/sacha/recordings/2024-01-19_08.26.35.webm?caption=Sending the current line]]

I used to do this kind of interaction with [[https://github.com/skeeto/skewer-mode][Skewer]], which also has some extra stuff for evaluating CSS and HTML. Skewer hasn't been updated in a while, but maybe I should also check that out again to see if I can get it working.

Anyway, now it's just a little bit easier to tinker with Javascript!

